==> sems-0.8.12.tar.gz: sems-0.8.12/plug-in/isdngw/IsdnUtils.cpp <==
/*
 * $Id: IsdnUtils.cpp,v 1.3.2.2 2004/01/23 13:40:58 ullstar Exp $
 *
 * Copyright (C) 2003 Ulrich Abend
 *
 * This file is part of isdngw, a free isdn gateway plugin 
 * for sems, a free SIP media server.
 *
 * isdngw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * isdngw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <termios.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include "globals.h"
#include "IsdnUtils.h"
#include "IsdnConfiguration.h"
#include "log.h"

/** timeout to wait for modem response in s/10 */
#define TTYI_TIMEOUT 5
/** max number of write retries */
#define WRITE_RETRY_LIMIT 10
/** max number of read retries, length of timeslots is def'd individual */
#define READ_RETRY_LIMIT 80


/** Nonblocking write a buffer to an open isdn device. The buffer must be 
 * a terminated c string.
 * @param isdn_fd open isdn device file handle
 * @param buf the buffer to be written
 * @return OK if the complete buffer is written <BR>
 * @return FAIL if the buffer could not be written completely
 */
int IsdnUtils::nb_write(int isdn_fd, char* buf) {
	int wlen;
	int retry = 0;
	int offset = 0;
	int write_count = 0;

	/* loop for writing */
	while (1) {
		if (retry == WRITE_RETRY_LIMIT) {
			ERROR("error while talking to isdn device: write retry limit reached\n");
			return (FAIL);
		}	
		/* try to write out buffer */
		wlen = strlen(buf);
		write_count = write(isdn_fd, buf + offset, wlen - offset);
		if (write_count == -1) {
			if (errno == EAGAIN) {
				retry++;
				continue;
			}
			ERROR("error while talking to isdn device: write failed, %s\n", strerror(errno));
			return (1);
		}
		if (write_count != wlen - offset) {
			DBG("partial write, only %i bytes written,\n", write_count);
			offset += write_count;
			continue;
		}
		/* no error, complete command written */
		break;
	} /* write loop */
	return (OK);
}

/** Reads an answer from an isdn device. This method polls for an 
 * answer from the isdn device, an answer is complete if it is terminated
 * by a CR and a LF. The answer is stored as terminated string in buf on success. 
 * Answers have a maximum length of 100 characters.
 * @param isdn_fd open isdn device file handle
 * @param buf buffer that will contain the result
 * @param options special options to consider, @see IsdnChat for details
 * @param custom_timeout sets a custom timeout for this answer in sec/10
 * @return FAIL on error <BR>OK on success <BR> -2 on timeout
 */
int IsdnUtils::nb_read_answer(int isdn_fd, char* buf, char options, int custom_timeout) {
/* read buffer size */
	int retry = 0;
	int offset = 0;
	int read_count;
	fd_set my_set; /* for use with select */
	struct timeval timeout;
	int select_ret;
	int ttyi_timeout = TTYI_TIMEOUT;
	
	if (custom_timeout > 0)
		ttyi_timeout = custom_timeout;
	
	/* read loop */
	while (1) {
		
		/* check if we tried hard enough to get an answer */
		if (retry >= READ_RETRY_LIMIT) {
#ifdef ISDNGW_EXTRA_DEBUG
			ERROR("error while talking to isdn device: read retry limit reached\n");
#endif
			return (-2);
		}

		/* construct everything needed for select */
		FD_ZERO(&my_set);
		FD_SET(isdn_fd, &my_set);
		/* set timeout */
		timeout.tv_sec = 0;
		timeout.tv_usec = ttyi_timeout * 100000 / READ_RETRY_LIMIT;  /* run maximum timeout in 20 runs */

		/* wait for input from modem */
		select_ret = select (isdn_fd+1, &my_set, NULL, NULL, &timeout);

		/* input from modem -> everything's fine */
		if (FD_ISSET(isdn_fd, &my_set)) { 
			/* try to read */
			read_count = read(isdn_fd, buf + offset, BUFSIZE - offset);
			/* there must be something broken, if we get -1 after select&ISSET */
			if (read_count == -1) {
				ERROR("error while talking to isdn device: read failed, %s\n", strerror(errno));
				return (FAIL);
			}
			/* terminate the string for further processing */
			*(buf+offset+read_count)='\0';
		
			/* deal with special options */
			if (options & OPTION_END_VOICE) {
				tcdrain(isdn_fd);
				//DBG("ending voice connection\n");
				/* finished */
				break;
			}	
			
			/* check if response is complete ( =ending with \10\13 ) */
			if (complete_response(buf)) { 
				/* only care about the last answer, strip cr's & lf's */
				if (! (options & OPTION_DONT_STRIP))
					stripresult(buf);
				/* finshed */ 
				break;
			}	else {
				/* read more bytes */
				offset += read_count;
				retry++;
				continue; 
			}	
		} else { /* ISSET after select */
			/* select ended with timeout \todo check if select is not interrupted by signal */
			retry++;
			continue;
		}
	} /* while */
	return (OK);
}	

/** Checks if a string a valid response. A valid isdn response ends on CR LF. 
 * @param buf null terminated buffer to be checked.
 * @return 0 on incomplete answers <BR> 1 on complete answers
 */
int IsdnUtils::complete_response (char* buf) {
	while ( *(buf++) != '\0' ) {
		if ((*buf == 13) && (*(buf+1) == 10))
			return 1;
	}		
	return 0;		
}

/** Removes linefeed and cr characters from string.
 * Input of type "tralla\13\10moretralla\13\10" results in "moretralla",
 * only the last response is of interest in our case.
 * @param buf terminated buffer to be stripped
 */
void IsdnUtils::stripresult (char* buf) {
	char* local;
	char* in;
	char* local_start;
	char c;
	int offset_last_lf;
	offset_last_lf = 0;
	in = buf;
	local = (char*) malloc (strlen(buf)+1);
	local_start = local;
	if (local == NULL) {
		ERROR("out of memory");
		exit(1);
	}
	while ( (c = *(in++)) != '\0' ) {
		if ((c != 13) && (c != 10))
			*(local++) = c;
		if ((c == 10) && (*in != '\0')) {  /* there is more after \n */
			*local = '\0';
			local = local_start;
		}	
	}
	*local = '\0';
	strcpy(buf, local_start);
	free(local_start);
};	
	
/** Takes a modem ring signaling and extracts number. E.g.: 
 * "CALLER NUMBER: <telnumber>\n\fRING\n\f" -> "<telnumber>"
 * @param buf modem answer
 * @return OK on success<BR>
 * @return FAIL on parse error
 */
int IsdnUtils::extractNumberOnCall(char* buf, char* callednumberbuf) {
	char* cur = buf;
	char* temp = buf;
	#define CALLER_NUMBER_STRING "CALLER NUMBER: "
	#define CALLER_NUMBER_LENGTH 15
	#define CALLED_NUMBER_LENGTH 5
	cur+=2; // skip cr+lf at the beginning
	if (strncmp(CALLER_NUMBER_STRING, cur, CALLER_NUMBER_LENGTH) != 0) {
		ERROR("could not parse modem call input %s\n", buf);
		return FAIL;
	}
	cur+=CALLER_NUMBER_LENGTH; /* now at start of caller number */
	while ((*(temp++) = *(cur++)) != '\n'); /* copy to caller array */
	*(temp-2)='\0'; /* terminate string instead of \n, caller parsed */
	cur += CALLED_NUMBER_LENGTH;
	temp = callednumberbuf;
	while ((*(temp++) = *(cur++)) != '\n'); /* copy to called array */
	*(temp-2)='\0'; /* terminate string instead of \n, called number parsed */
	//DBG("extract: call from >%s< to >%s<\n", buf, callednumberbuf);
	return OK;
}

void IsdnUtils::determineNumbers(AmRequest *req, char* phonenumber, char* msn, int buflens) {
	determineNumbers(req->cmd, phonenumber, msn, buflens);
}

void IsdnUtils::determineNumbers(AmCmd cmd, char* phonenumber, char* msn, int buflens) {
	/* create space for the phonenumber and fill it */
	strncpy (phonenumber, cmd.user.c_str(), buflens);

	/* check if phonenumber is allowed to be called */
	char c;
	char* foo;
	foo = phonenumber;
	while ((c = *(foo++)) != '\0') {
		if ( (c < '0') || (c > '9') ) {
			throw AmSession::Exception(500,"isdngw: illegal number");		
		}
	}	

	//INFO("determineNumbers: calling number: %s\n", phonenumber);

	if (strlen(phonenumber) < isdn_config->minnumberlen) {
		free(phonenumber);
		throw AmSession::Exception(500,"isdngw: number too short");
	}	
	
	/* the number is ok, try to place the call */
	/* check outgoing msn first */
	const char* sip_from = cmd.from.c_str();
	char temp[201];
	char* tmp = temp;
	*tmp = '\0';
	/* just to make shure */
	if (strlen(sip_from) > 200) {
		ERROR("ERROR: ignoring too long from header (>200), using default msn instead\n");
	} else if (isdn_config->msnsipcaller) {
		while (*(sip_from++) != ':');
		if (*sip_from == '+')
			sip_from++;	
		while (1) {
			c = (*(tmp++) = *(sip_from++));
			if (c == '@') {
				*(tmp-1)='\0';
				break;
			}	
			if ((c < '0') || (c > '9')) {
				*temp = '\0';
				break;
			}
		}	
		if (*temp == '\0') 
			INFO("WARNING: SIP from header %s contains no valid phone number, using default instead\n", cmd.from.c_str());
	}
	strcpy(msn, temp);
}

#define SIP_PREFIX "sip:"
#define SIP_PREFIX_LEN 4
void IsdnUtils::buildSipUri(char* buf, int len, char* number, bool caller) {
	char* domain;
	int domainlen;
	domain = caller?isdn_config->callerdomain:isdn_config->calleedomain;
	domainlen = strlen(domain);
	
	if (SIP_PREFIX_LEN + strlen(number) + domainlen + 2 > (unsigned)len) {
		ERROR("could not build sip uri, not enough space in buffer\n");
	} else {
		strcpy(buf, SIP_PREFIX);
		strcat(buf, number);
		strcat(buf, "@");
		strcat(buf, domain);
	}
}

==> sems_2004_01_04.tar.gz: sems_2004_01_04/plug-in/isdngw/IsdnUtils.cpp <==
/*
 * $Id: IsdnUtils.cpp,v 1.3 2003/11/16 19:30:34 ullstar Exp $
 *
 * Copyright (C) 2003 Ulrich Abend
 *
 * This file is part of isdngw, a free isdn gateway plugin 
 * for sems, a free SIP media server.
 *
 * isdngw is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * isdngw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <termios.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include "globals.h"
#include "IsdnUtils.h"
#include "IsdnConfiguration.h"
#include "log.h"

/** timeout to wait for modem response in s/10 */
#define TTYI_TIMEOUT 5
/** max number of write retries */
#define WRITE_RETRY_LIMIT 10
/** max number of read retries, length of timeslots is def'd individual */
#define READ_RETRY_LIMIT 80
/** max number length */
#define MAX_NUM_LEN 20


/** Nonblocking write a buffer to an open isdn device. The buffer must be 
 * a terminated c string.
 * @param isdn_fd open isdn device file handle
 * @param buf the buffer to be written
 * @return OK if the complete buffer is written <BR>
 * @return FAIL if the buffer could not be written completely
 */
int IsdnUtils::nb_write(int isdn_fd, char* buf) {
	int wlen;
	int retry = 0;
	int offset = 0;
	int write_count = 0;

	/* loop for writing */
	while (1) {
		if (retry == WRITE_RETRY_LIMIT) {
			ERROR("error while talking to isdn device: write retry limit reached\n");
			return (FAIL);
		}	
		/* try to write out buffer */
		wlen = strlen(buf);
		write_count = write(isdn_fd, buf + offset, wlen - offset);
		if (write_count == -1) {
			if (errno == EAGAIN) {
				retry++;
				continue;
			}
			ERROR("error while talking to isdn device: write failed, %s\n", strerror(errno));
			return (1);
		}
		if (write_count != wlen - offset) {
			DBG("partial write, only %i bytes written,\n", write_count);
			offset += write_count;
			continue;
		}
		/* no error, complete command written */
		break;
	} /* write loop */
	return (OK);
}

/** Reads an answer from an isdn device. This method polls for an 
 * answer from the isdn device, an answer is complete if it is terminated
 * by a CR and a LF. The answer is stored as terminated string in buf on success. 
 * Answers have a maximum length of 100 characters.
 * @param isdn_fd open isdn device file handle
 * @param buf buffer that will contain the result
 * @param options special options to consider, @see IsdnChat for details
 * @param custom_timeout sets a custom timeout for this answer in sec/10
 * @return FAIL on error <BR>OK on success <BR> -2 on timeout
 */
int IsdnUtils::nb_read_answer(int isdn_fd, char* buf, char options, int custom_timeout) {
/* read buffer size */
	int retry = 0;
	int offset = 0;
	int read_count;
	fd_set my_set; /* for use with select */
	struct timeval timeout;
	int select_ret;
	int ttyi_timeout = TTYI_TIMEOUT;
	
	if (custom_timeout > 0)
		ttyi_timeout = custom_timeout;
	
	/* read loop */
	while (1) {
		
		/* check if we tried hard enough to get an answer */
		if (retry >= READ_RETRY_LIMIT) {
#ifdef ISDNGW_EXTRA_DEBUG
			ERROR("error while talking to isdn device: read retry limit reached\n");
#endif
			return (-2);
		}

		/* construct everything needed for select */
		FD_ZERO(&my_set);
		FD_SET(isdn_fd, &my_set);
		/* set timeout */
		timeout.tv_sec = 0;
		timeout.tv_usec = ttyi_timeout * 100000 / READ_RETRY_LIMIT;  /* run maximum timeout in 20 runs */

		/* wait for input from modem */
		select_ret = select (isdn_fd+1, &my_set, NULL, NULL, &timeout);

		/* input from modem -> everything's fine */
		if (FD_ISSET(isdn_fd, &my_set)) { 
			/* try to read */
			read_count = read(isdn_fd, buf + offset, BUFSIZE - offset);
			/* there must be something broken, if we get -1 after select&ISSET */
			if (read_count == -1) {
				ERROR("error while talking to isdn device: read failed, %s\n", strerror(errno));
				return (FAIL);
			}
			/* terminate the string for further processing */
			*(buf+offset+read_count)='\0';
		
			/* deal with special options */
			if (options & OPTION_END_VOICE) {
				tcdrain(isdn_fd);
				DBG("ending voice connection\n");
				/* finished */
				break;
			}	
			
			/* check if response is complete ( =ending with \10\13 ) */
			if (complete_response(buf)) { 
				/* only care about the last answer, strip cr's & lf's */
				if (! (options & OPTION_DONT_STRIP))
					stripresult(buf);
				/* finshed */ 
				break;
			}	else {
				/* read more bytes */
				offset += read_count;
				retry++;
				continue; 
			}	
		} else { /* ISSET after select */
			/* select ended with timeout \todo check if select is not interrupted by signal */
			retry++;
			continue;
		}
	} /* while */
	return (OK);
}	

/** Checks if a string a valid response. A valid isdn response ends on CR LF. 
 * @param buf null terminated buffer to be checked.
 * @return 0 on incomplete answers <BR> 1 on complete answers
 */
int IsdnUtils::complete_response (char* buf) {
	while ( *(buf++) != '\0' ) {
		if ((*buf == 13) && (*(buf+1) == 10))
			return 1;
	}		
	return 0;		
}

/** Removes linefeed and cr characters from string.
 * Input of type "tralla\13\10moretralla\13\10" results in "moretralla",
 * only the last response is of interest in our case.
 * @param buf terminated buffer to be stripped
 */
void IsdnUtils::stripresult (char* buf) {
	char* local;
	char* in;
	char* local_start;
	char c;
	int offset_last_lf;
	offset_last_lf = 0;
	in = buf;
	local = (char*) malloc (strlen(buf)+1);
	local_start = local;
	if (local == NULL) {
		ERROR("out of memory");
		exit(1);
	}
	while ( (c = *(in++)) != '\0' ) {
		if ((c != 13) && (c != 10))
			*(local++) = c;
		if ((c == 10) && (*in != '\0')) {  /* there is more after \n */
			*local = '\0';
			local = local_start;
		}	
	}
	*local = '\0';
	strcpy(buf, local_start);
	free(local_start);
};	
	
/** Takes a modem ring signaling and extracts number. E.g.: 
 * "CALLER NUMBER: <telnumber>\n\fRING\n\f" -> "<telnumber>"
 * @param buf modem answer
 * @return OK on success<BR>
 * @return FAIL on parse error
 */
int IsdnUtils::extractNumberOnCall(char* buf, char* callednumberbuf) {
	char* cur = buf;
	char* temp = buf;
	#define CALLER_NUMBER_STRING "CALLER NUMBER: "
	#define CALLER_NUMBER_LENGTH 15
	#define CALLED_NUMBER_LENGTH 5
	cur+=2; // skip cr+lf at the beginning
	if (strncmp(CALLER_NUMBER_STRING, cur, CALLER_NUMBER_LENGTH) != 0) {
		ERROR("could not parse modem call input %s\n", buf);
		return FAIL;
	}
	cur+=CALLER_NUMBER_LENGTH; /* now at start of caller number */
	while ((*(temp++) = *(cur++)) != '\n'); /* copy to caller array */
	*(temp-2)='\0'; /* terminate string instead of \n, caller parsed */
	cur += CALLED_NUMBER_LENGTH;
	temp = callednumberbuf;
	while ((*(temp++) = *(cur++)) != '\n'); /* copy to called array */
	*(temp-2)='\0'; /* terminate string instead of \n, called number parsed */
	//DBG("extract: call from >%s< to >%s<\n", buf, callednumberbuf);
	return OK;
}

void IsdnUtils::determineNumbers(AmRequest *req, char* phonenumber, char* msn, int buflens) {
	determineNumbers(req->cmd, phonenumber, msn, buflens);
}

void IsdnUtils::determineNumbers(AmCmd cmd, char* phonenumber, char* msn, int buflens) {
	/* create space for the phonenumber and fill it */
	strncpy (phonenumber, cmd.user.c_str(), buflens);

	/* check if phonenumber is allowed to be called */
	char c;
	char* foo;
	foo = phonenumber;
	while ((c = *(foo++)) != '\0') {
		if ( (c < '0') || (c > '9') ) {
			throw AmSession::Exception(500,"isdngw: illegal number");		
		}
	}	

	INFO("determineNumbers: calling number: %s\n", phonenumber);

	if (strlen(phonenumber) < isdn_config->minnumberlen) {
		free(phonenumber);
		throw AmSession::Exception(500,"isdngw: number too short");
	}	
	
	/* the number is ok, try to place the call */
	/* check outgoing msn first */
	const char* sip_from = cmd.from.c_str();
	char temp[200];
	char* tmp = temp;
	*tmp = '\0';
	/* just to make shure */
	if (strlen(sip_from) > 200) {
		ERROR("ERROR: ignoring too long from header (>200), using default msn instead\n");
	} else if (isdn_config->msnsipcaller) {
		while (*(sip_from++) != ':');
		if (*sip_from == '+')
			sip_from++;	
		while (1) {
			c = (*(tmp++) = *(sip_from++));
			if (c == '@') {
				*(tmp-1)='\0';
				break;
			}	
			if ((c < '0') || (c > '9')) {
				*temp = '\0';
				break;
			}
		}	
		if (*temp == '\0') 
			INFO("WARNING: SIP from header %s contains no valid phone number, using default instead\n", cmd.from.c_str());
	}
	strcpy(msn, temp);
}
